"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildNgssc = exports.detectVariables = exports.detectVariablesAndBuildNgsscJson = exports.ngsscBuild = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const architect_1 = require("@angular-devkit/architect");
const variable_detector_1 = require("./variable-detector");
const readFileAsync = fs_1.promises.readFile;
const writeFileAsync = fs_1.promises.writeFile;
function ngsscBuild(options, context) {
    return __awaiter(this, void 0, void 0, function* () {
        const browserTarget = (0, architect_1.targetFromTargetString)(options.browserTarget);
        const rawBrowserOptions = yield context.getTargetOptions(browserTarget);
        const browserName = yield context.getBuilderNameForTarget(browserTarget);
        const browserOptions = yield context.validateOptions(rawBrowserOptions, browserName);
        const scheduledTarget = yield context.scheduleTarget(browserTarget);
        const result = yield scheduledTarget.result;
        yield detectVariablesAndBuildNgsscJson(options, browserOptions, context);
        return result;
    });
}
exports.ngsscBuild = ngsscBuild;
function detectVariablesAndBuildNgsscJson(options, browserOptions, context, multiple = false) {
    return __awaiter(this, void 0, void 0, function* () {
        const ngsscContext = yield detectVariables(options, context);
        const outputPath = (0, path_1.join)(context.workspaceRoot, browserOptions.outputPath);
        const ngssc = buildNgssc(ngsscContext, options, browserOptions, multiple);
        yield writeFileAsync((0, path_1.join)(outputPath, 'ngssc.json'), JSON.stringify(ngssc, null, 2), 'utf8');
    });
}
exports.detectVariablesAndBuildNgsscJson = detectVariablesAndBuildNgsscJson;
function detectVariables(options, context) {
    return __awaiter(this, void 0, void 0, function* () {
        const environmentVariableFile = (0, path_1.join)(context.workspaceRoot, options.ngsscEnvironmentFile);
        const detector = new variable_detector_1.VariableDetector(context.logger);
        const fileContent = yield readFileAsync(environmentVariableFile, 'utf8');
        const ngsscContext = detector.detect(fileContent);
        context.logger.info(`ngssc: Detected variant '${ngsscContext.variant}' with variables ` +
            `'${ngsscContext.variables.join(', ')}'`);
        if (ngsscContext.variant === 'NG_ENV') {
            context.logger.warn('Variant NG_ENV is deprecated and will be removed with version 14. ' +
                'Please change usage to `process.env`.');
        }
        return ngsscContext;
    });
}
exports.detectVariables = detectVariables;
function buildNgssc(ngsscContext, options, browserOptions, multiple = false) {
    return {
        environmentVariables: [
            ...ngsscContext.variables,
            ...(options.additionalEnvironmentVariables || []),
        ],
        filePattern: options.filePattern || extractFilePattern(browserOptions === null || browserOptions === void 0 ? void 0 : browserOptions.index, multiple),
        variant: ngsscContext.variant,
    };
}
exports.buildNgssc = buildNgssc;
function extractFilePattern(index, multiple) {
    let result = '**/index.html';
    if (!index) {
        return result;
    }
    else if (typeof index === 'string') {
        result = (0, path_1.basename)(index);
    }
    else if (index.output) {
        result = (0, path_1.basename)(index.output);
    }
    else {
        result = (0, path_1.basename)(index.input);
    }
    return multiple && !result.startsWith('*') ? `**/${result}` : result;
}
exports.default = (0, architect_1.createBuilder)(ngsscBuild);
//# sourceMappingURL=index.js.map