"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ngAdd = void 0;
const core_1 = require("@angular-devkit/core");
const schematics_1 = require("@angular-devkit/schematics");
const change_1 = require("@schematics/angular/utility/change");
const workspace_1 = require("@schematics/angular/utility/workspace");
function ngAdd(options) {
    return (0, schematics_1.chain)([
        addNgsscTargetToWorkspace(options),
        addImportAndDescriptionToEnvironmentFile(options),
        addNgsscToPackageScripts(options),
        addPlaceholderToIndexHtml(options),
    ]);
}
exports.ngAdd = ngAdd;
function addNgsscTargetToWorkspace(options) {
    return (_host, context) => (0, workspace_1.updateWorkspace)((workspace) => {
        const project = workspace.projects.get(options.project);
        if (!project) {
            return;
        }
        const ngsscOptions = {
            additionalEnvironmentVariables: options.additionalEnvironmentVariables
                ? options.additionalEnvironmentVariables.split(',').map((e) => e.trim())
                : [],
            ngsscEnvironmentFile: (0, core_1.join)((0, core_1.normalize)(project.root), options.ngsscEnvironmentFile),
        };
        if (options.experimentalBuilders) {
            const buildTarget = project.targets.get('build');
            const { production, development } = buildTarget.configurations || {};
            if (production && development) {
                const fileReplacements = production['fileReplacements'];
                const devEnvironmentFile = Array.isArray(fileReplacements) &&
                    fileReplacements.find((f) => f.with === ngsscOptions.ngsscEnvironmentFile);
                if (devEnvironmentFile) {
                    development['ngsscEnvironmentFile'] = devEnvironmentFile.replace;
                }
            }
            buildTarget.builder = 'angular-server-side-configuration:browser';
            buildTarget.options = Object.assign(Object.assign({}, buildTarget.options), ngsscOptions);
            const serveTarget = project.targets.get('serve');
            serveTarget.builder = 'angular-server-side-configuration:dev-server';
            const target = project.targets.get('ngsscbuild');
            if (target) {
                project.targets.delete('ngsscbuild');
            }
            return;
        }
        const target = project.targets.get('ngsscbuild');
        if (target) {
            context.logger.info(`Skipping adding ngsscbuild target to angular.json, as it already exists in project ${options.project}.`);
            return;
        }
        project.targets.add({
            name: 'ngsscbuild',
            builder: 'angular-server-side-configuration:ngsscbuild',
            options: Object.assign(Object.assign({}, ngsscOptions), { browserTarget: `${options.project}:build` }),
            configurations: {
                production: {
                    browserTarget: `${options.project}:build:production`,
                },
            },
        });
    });
}
function addImportAndDescriptionToEnvironmentFile(options) {
    return (host, context) => __awaiter(this, void 0, void 0, function* () {
        const { project } = yield resolveWorkspace(options, host);
        const normalizedPath = (0, core_1.join)((0, core_1.normalize)(project.root), options.ngsscEnvironmentFile);
        const file = host.get(normalizedPath);
        if (!file) {
            throw new schematics_1.SchematicsException(`${normalizedPath} does not exist!`);
        }
        else if (file.content.includes('angular-server-side-configuration')) {
            context.logger.info(`Skipping adding import to ${file.path}, since import was already detected.`);
            return;
        }
        const insertContent = `import 'angular-server-side-configuration/process';

/**
 * How to use angular-server-side-configuration:
 *
 * Use process.env['NAME_OF_YOUR_ENVIRONMENT_VARIABLE']
 *
 * export const environment = {
 *   stringValue: process.env['STRING_VALUE'],
 *   stringValueWithDefault: process.env['STRING_VALUE'] || 'defaultValue',
 *   numberValue: Number(process.env['NUMBER_VALUE']),
 *   numberValueWithDefault: Number(process.env['NUMBER_VALUE'] || 10),
 *   booleanValue: Boolean(process.env['BOOLEAN_VALUE']),
 *   booleanValueInverted: process.env['BOOLEAN_VALUE_INVERTED'] !== 'false',
 * };
 * 
 * Please note that process.env[variable] cannot be resolved. Please directly use strings.
 */

`;
        const insertion = new change_1.InsertChange(file.path, 0, insertContent);
        const recorder = host.beginUpdate(file.path);
        recorder.insertLeft(insertion.pos, insertion.toAdd);
        host.commitUpdate(recorder);
    });
}
function addNgsscToPackageScripts(options) {
    return (host, context) => {
        if (options.experimentalBuilders) {
            return;
        }
        const pkgPath = '/package.json';
        const buffer = host.read(pkgPath);
        if (buffer === null) {
            throw new schematics_1.SchematicsException('Could not find package.json');
        }
        const pkg = Object.assign({ scripts: {} }, JSON.parse(buffer.toString()));
        if ('build:ngssc' in pkg.scripts) {
            context.logger.info(`Skipping adding script to package.json, as it already exists.`);
            return;
        }
        pkg.scripts['build:ngssc'] = `ng run ${options.project}:ngsscbuild:production`;
        host.overwrite(pkgPath, JSON.stringify(pkg, null, 2));
    };
}
function addPlaceholderToIndexHtml(options) {
    return (host, context) => __awaiter(this, void 0, void 0, function* () {
        var _a;
        const { project } = yield resolveWorkspace(options, host);
        const build = project.targets.get('build');
        if (!build) {
            throw new schematics_1.SchematicsException(`Expected a build target in project ${options.project}!`);
        }
        const indexPath = ((_a = build.options) === null || _a === void 0 ? void 0 : _a['index']) || 'src/index.html';
        const indexHtml = host.get(indexPath);
        if (!indexHtml) {
            throw new schematics_1.SchematicsException(`Expected index html ${indexPath} to exist!`);
        }
        const indexHtmlContent = indexHtml.content.toString();
        if (/<!--\s*CONFIG\s*-->/.test(indexHtmlContent)) {
            context.logger.info(`Skipping adding placeholder to ${indexHtml.path}, as it already contains it.`);
            return;
        }
        const insertIndex = indexHtmlContent.includes('</title>')
            ? indexHtmlContent.indexOf('</title>') + 9
            : indexHtmlContent.indexOf('</head>');
        const insertion = new change_1.InsertChange(indexHtml.path, insertIndex, '  <!--CONFIG-->\n');
        const recorder = host.beginUpdate(indexHtml.path);
        recorder.insertLeft(insertion.pos, insertion.toAdd);
        host.commitUpdate(recorder);
    });
}
function resolveWorkspace(options, host) {
    return __awaiter(this, void 0, void 0, function* () {
        const workspace = yield (0, workspace_1.getWorkspace)(host);
        const project = workspace.projects.get(options.project);
        if (!project) {
            throw new schematics_1.SchematicsException(`Project ${options.project} not found!`);
        }
        return { workspace, project };
    });
}
//# sourceMappingURL=index.js.map