"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dockerfile = exports.updateToV9 = exports.updateToV8 = void 0;
const core_1 = require("@angular-devkit/core");
const schematics_1 = require("@angular-devkit/schematics");
const workspace_1 = require("@schematics/angular/utility/workspace");
const index_1 = require("../ng-add/index");
const NGSSC_JSON_PATH = '/ngssc.json';
function updateToV8() {
    return (tree) => __awaiter(this, void 0, void 0, function* () {
        const ngssc = tryReadNgsscJson(tree);
        const workspace = yield (0, workspace_1.getWorkspace)(tree);
        const projects = Array.from(workspace.projects.keys());
        return (0, schematics_1.chain)([
            ...projects.map((project) => (0, index_1.ngAdd)({
                additionalEnvironmentVariables: (ngssc.environmentVariables || []).join(','),
                experimentalBuilders: false,
                ngsscEnvironmentFile: 'src/environments/environment.prod.ts',
                project,
            })),
            removeNgsscJson(),
            checkNgsscUsageInScripts(),
        ]);
    });
}
exports.updateToV8 = updateToV8;
function updateToV9() {
    return (_tree, context) => {
        return (0, workspace_1.updateWorkspace)((workspace) => {
            context.logger.info(`Removing ngsscbuild entry 'aotSupport', since it is no longer necessary for Ivy.`);
            workspace.projects.forEach((project, name) => {
                const ngsscbuild = project.targets.get('ngsscbuild');
                if (!ngsscbuild || !ngsscbuild.options) {
                    return;
                }
                if ('aotSupport' in ngsscbuild.options) {
                    delete ngsscbuild.options['aotSupport'];
                    context.logger.info(` - Removed from ${name} ngsscbuild options`);
                }
                Object.keys(ngsscbuild.configurations || {})
                    .filter((c) => 'aotSupport' in ngsscbuild.configurations[c])
                    .forEach((c) => {
                    delete ngsscbuild.configurations[c]['aotSupport'];
                    context.logger.info(` - Removed from ${name} ngsscbuild configuration ${c}`);
                });
            });
        });
    };
}
exports.updateToV9 = updateToV9;
function dockerfile() {
    return (tree) => {
        const downloadUrlRegex = /https:\/\/github.com\/kyubisation\/angular-server-side-configuration\/releases\/download\/v((0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?)/;
        const version = require('../../package.json').version;
        tree.visit((path, entry) => {
            if ((0, core_1.basename)(path).indexOf('Dockerfile') >= 0 &&
                entry &&
                entry.content.toString().match(downloadUrlRegex)) {
                const content = entry.content
                    .toString()
                    .replace(new RegExp(downloadUrlRegex.source, 'g'), `https://github.com/kyubisation/angular-server-side-configuration/releases/download/v${version}`);
                tree.overwrite(path, content);
            }
        });
    };
}
exports.dockerfile = dockerfile;
function tryReadNgsscJson(tree) {
    const ngssc = tree.read(NGSSC_JSON_PATH);
    return ngssc ? JSON.parse(ngssc.toString('utf8')) : {};
}
function removeNgsscJson() {
    return (tree, context) => {
        if (tree.exists(NGSSC_JSON_PATH)) {
            tree.delete(NGSSC_JSON_PATH);
            context.logger.info('Removed legacy ngssc.json');
        }
    };
}
function checkNgsscUsageInScripts() {
    return (tree, context) => __awaiter(this, void 0, void 0, function* () {
        const packageJson = tree.read('/package.json');
        if (!packageJson) {
            return;
        }
        const pkg = JSON.parse(packageJson.toString('utf8'));
        const ngsscUsedInScripts = Object.keys(pkg.scripts || {}).some((k) => pkg.scripts[k].includes('ngssc '));
        if (!ngsscUsedInScripts) {
            return;
        }
        const workspace = yield (0, workspace_1.getWorkspace)(tree);
        const projectName = Array.from(workspace.projects.keys())[0];
        context.logger.info('Please remove the ngssc usage from your scripts.');
        context.logger.info(`To run the ngssc build, run the command \`ng run ${projectName}:ngsscbuild:production\`.`);
    });
}
//# sourceMappingURL=index.js.map